/* light.css */
@media screen {
    body { color: #000; background: #ffe; }
    strong { background: #ff0; }
    :link { color: #1e4c43; }
    :visited { color: #1e354c; }
    em, strong, .em { color: #4c0f0f; }
    code, .code { color: #0f4c0f; }
    dfn { color: #4c0f4c; }

    blockquote { background: #ffff00; }
    li.new1 { background: #ffc233; }
    li.new2 { background: #c2ff33; }

    pre.docsrc { background: #b3ffff; }
    .kbd { color: #0f4c0f; background: #ffb3ff; }

    .regexp { background: #99ff33; }
    .regexp1 { background: #33ff99; }
    .magic { background: #ffffff; }

    div#toc, .toc { background: #f0b3ff; }
    div#toc a:hover { background: #ffffff; }
    div#toc a:hover .hovertext { color: #0f4c0f; }
    div#toc a:hover .hovertext em { background: #eb33ff; }
}
}
